@@ -1,48 +1,39 @@
-import os
-
 try:
-    from gi import Repository
-    if Repository.get_default().enumerate_versions('GnomeKeyring'):
-        from gi.repository import GnomeKeyring
-except ImportError:
+    import gi
+    gi.require_version('GnomeKeyring', '1.0')
+    from gi.repository import GnomeKeyring
+except (ImportError, ValueError):
     pass
 
-from keyring.backend import KeyringBackend
-from keyring.errors import PasswordSetError, PasswordDeleteError
-from keyring.util import properties
-from keyring.util import XDG
-from keyring.py27compat import unicode_str
+from ..backend import KeyringBackend
+from ..errors import PasswordSetError, PasswordDeleteError
+from ..util import properties
+from ..py27compat import unicode_str
+
 
 class Keyring(KeyringBackend):
     """Gnome Keyring"""
 
-    # Name of the keyring to store the passwords in.
-    # Use None for the default keyring.
     KEYRING_NAME = None
-
-    requisite_vars = [
-        'GNOME_KEYRING_CONTROL',
-        'DISPLAY',
-        'DBUS_SESSION_BUS_ADDRESS',
-    ]
+    """
+    Name of the keyring in which to store the passwords.
+    Use None for the default keyring.
+    """
 
     @properties.ClassProperty
     @classmethod
-    @XDG.Preference('Gnome')
     def priority(cls):
         if 'GnomeKeyring' not in globals():
             raise RuntimeError("GnomeKeyring module required")
-        if not cls.has_requisite_vars():
-            raise RuntimeError("Requisite environment vars are not present")
-        return int(cls.has_requisite_vars())
+        result = GnomeKeyring.get_default_keyring_sync()[0]
+        if result != GnomeKeyring.Result.OK:
+            raise RuntimeError(result.value_name)
+        return 1
 
-    @classmethod
-    def has_requisite_vars(cls):
-        """
-        Return True if the requisite environment vars are present in the
-        environment.
-        """
-        return set(cls.requisite_vars).issubset(os.environ)
+    @property
+    def keyring_name(self):
+        system_default = GnomeKeyring.get_default_keyring_sync()[1]
+        return self.KEYRING_NAME or system_default
 
     def _find_passwords(self, service, username, deleting=False):
         """Get password of the username for the service
@@ -87,7 +78,7 @@ class Keyring(KeyringBackend):
         GnomeKeyring.Attribute.list_append_string(attrs, 'service', service)
         GnomeKeyring.Attribute.list_append_string(attrs, 'application', 'python-keyring')
         result = GnomeKeyring.item_create_sync(
-            self.KEYRING_NAME, GnomeKeyring.ItemType.NETWORK_PASSWORD,
+            self.keyring_name, GnomeKeyring.ItemType.NETWORK_PASSWORD,
             "Password for '%s' on '%s'" % (username, service),
             attrs, password, True)[0]
         if result == GnomeKeyring.Result.CANCELLED:
