@@ -1,9 +1,8 @@
 import logging
 
-from keyring.util import properties
-from keyring.util import XDG
-from keyring.backend import KeyringBackend
-from keyring.errors import (InitError, PasswordDeleteError,
+from ..util import properties
+from ..backend import KeyringBackend
+from ..errors import (InitError, PasswordDeleteError,
     ExceptionRaisedContext)
 
 try:
@@ -19,7 +18,6 @@ class Keyring(KeyringBackend):
 
     @properties.ClassProperty
     @classmethod
-    @XDG.Preference('Gnome')
     def priority(cls):
         with ExceptionRaisedContext() as exc:
             secretstorage.__name__
