@@ -1,18 +1,21 @@
 """
-core.py
+Core API functions and initialization routines.
+"""
 
-Created by Kang Zhang on 2009-07-09
-"""
 import os
 import sys
-import warnings
 import logging
+import operator
 
-from .py27compat import configparser
+from .py27compat import configparser, filter
+from .py33compat import max
 
 from . import logger
 from . import backend
 from .util import platform_ as platform
+from .util import once
+from .backends import fail
+
 
 log = logging.getLogger(__name__)
 
@@ -51,92 +54,91 @@ def delete_password(service_name, username):
     _keyring_backend.delete_password(service_name, username)
 
 
-def init_backend():
+recommended = lambda backend: backend.priority >= 1
+by_priority = operator.attrgetter('priority')
+
+
+def init_backend(limit=None):
     """
     Load a keyring specified in the config file or infer the best available.
     """
     _load_library_extensions()
-    set_keyring(load_config() or _get_best_keyring())
+    keyrings = filter(limit, backend.get_all_keyring())
+
+    set_keyring(
+        load_config()
+        or max(keyrings, default=fail.Keyring, key=by_priority)
+    )
 
 
-def _get_best_keyring():
+def _load_keyring_class(keyring_name):
     """
-    Return the best keyring backend for the given environment based on
-    priority.
+    Load the keyring class indicated by name.
+
+    These popular names are tested to ensure their presence.
+
+    >>> popular_names = [
+    ...      'keyring.backends.Windows.WinVaultKeyring',
+    ...      'keyring.backends.OS_X.Keyring',
+    ...      'keyring.backends.kwallet.DBusKeyring',
+    ...      'keyring.backends.SecretService.Keyring',
+    ...      'keyring.backends.Gnome.Keyring',
+    ...  ]
+    >>> list(map(_load_keyring_class, popular_names))
+    [...]
+
+    These legacy names are retained for compatibility.
+
+    >>> legacy_names = [
+    ...      'keyring.backends.kwallet.Keyring',
+    ...  ]
+    >>> list(map(_load_keyring_class, legacy_names))
+    [...]
     """
-    keyrings = backend.get_all_keyring()
-    # rank by priority
-    keyrings.sort(key = lambda x: -x.priority)
-    # get the most recommended one
-    return keyrings[0]
+    module_name, sep, class_name = keyring_name.rpartition('.')
+    __import__(module_name)
+    module = sys.modules[module_name]
+    return getattr(module, class_name)
 
 
-def load_keyring(keyring_path, keyring_name):
+def load_keyring(keyring_name):
     """
     Load the specified keyring by name (a fully-qualified name to the
     keyring, such as 'keyring.backends.file.PlaintextKeyring')
-
-    `keyring_path` is an additional, optional search path and may be None.
-    **deprecated** In the future, keyring_path must be None.
     """
-    module_name, sep, class_name = keyring_name.rpartition('.')
-    if keyring_path is not None and keyring_path not in sys.path:
-        warnings.warn("keyring_path is deprecated and should always be None",
-            DeprecationWarning)
-        sys.path.insert(0, keyring_path)
-    __import__(module_name)
-    module = sys.modules[module_name]
-    class_ = getattr(module, class_name)
+    class_ = _load_keyring_class(keyring_name)
     # invoke the priority to ensure it is viable, or raise a RuntimeError
     class_.priority
     return class_()
 
 
 def load_config():
-    """Load a keyring using the config file.
-
-    The config file can be in the current working directory, or in the user's
-    home directory.
-    """
-    keyring = None
+    """Load a keyring using the config file in the config root."""
 
     filename = 'keyringrc.cfg'
 
-    if os.path.exists(filename):
-        warnings.warn("Loading config from the current directory "
-            "(./keyringrc.cfg). This behavior is deprecated and will be "
-            "removed in a future version.")
-    local_path = os.path.join(os.getcwd(), filename)
-    config_path = os.path.join(platform.config_root(), filename)
+    keyring_cfg = os.path.join(platform.config_root(), filename)
 
-    # search from current working directory and the data root
-    keyring_cfg_candidates = [local_path, config_path]
+    if not os.path.exists(keyring_cfg):
+        return
 
-    # initialize the keyring_config with the first detected config file
-    keyring_cfg = None
-    for path in keyring_cfg_candidates:
-        keyring_cfg = path
-        if os.path.exists(path):
-            break
+    config = configparser.RawConfigParser()
+    config.read(keyring_cfg)
+    _load_keyring_path(config)
 
-    if os.path.exists(keyring_cfg):
-        config = configparser.RawConfigParser()
-        config.read(keyring_cfg)
-        _load_keyring_path(config)
+    # load the keyring class name, and then load this keyring
+    try:
+        if config.has_section("backend"):
+            keyring_name = config.get("backend", "default-keyring").strip()
+        else:
+            raise configparser.NoOptionError('backend', 'default-keyring')
 
-        # load the keyring class name, and then load this keyring
-        try:
-            if config.has_section("backend"):
-                keyring_name = config.get("backend", "default-keyring").strip()
-            else:
-                raise configparser.NoOptionError('backend', 'default-keyring')
+    except (configparser.NoOptionError, ImportError):
+        logger.warning("Keyring config file contains incorrect values.\n" +
+                       "Config file: %s" % keyring_cfg)
+        return
 
-            keyring = load_keyring(None, keyring_name)
-        except (configparser.NoOptionError, ImportError):
-            logger.warning("Keyring config file contains incorrect values.\n" +
-                           "Config file: %s" % keyring_cfg)
-
-    return keyring
+    return load_keyring(keyring_name)
 
 def _load_keyring_path(config):
     "load the keyring-path option (if present)"
@@ -146,6 +148,7 @@ def _load_keyring_path(config):
     except (configparser.NoOptionError, configparser.NoSectionError):
         pass
 
+@once
 def _load_library_extensions():
     """
     Locate all setuptools entry points by the name 'keyring backends'
